@@ -19,7 +19,10 @@ QVideoFrameTextures* QVideoFrameTexturePool::updateTextures(QRhi &rhi, QRhiResou
 
     m_texturesDirty = false;
     QVideoFrameTexturesUPtr &textures = m_textureSlots[currentSlot];
-    textures = QVideoTextureHelper::createTextures(m_currentFrame, rhi, rub, std::move(textures));
+    Q_ASSERT(!m_oldTextures);
+    m_oldTextures = std::move(textures);
+    textures = QVideoTextureHelper::createTextures(m_currentFrame, rhi, rub, m_oldTextures);
+
     m_currentSlot = textures ? currentSlot : std::optional<int>{};
 
     return textures.get();
@@ -27,6 +30,7 @@ QVideoFrameTextures* QVideoFrameTexturePool::updateTextures(QRhi &rhi, QRhiResou
 
 void QVideoFrameTexturePool::onFrameEndInvoked()
 {
+    m_oldTextures.reset();
     if (m_currentSlot && m_textureSlots[*m_currentSlot])
         m_textureSlots[*m_currentSlot]->onFrameEndInvoked();
 }
