@@ -15,10 +15,6 @@
 #include <QtMultimedia/private/qwindowsaudiosource_p.h>
 #include <QtMultimedia/private/qwindows_propertystore_p.h>
 
-#if defined(Q_CC_MINGW) && !defined(INITGUID)
-// mingw's uuid.lib does not contain PKEY_Device_FriendlyName and friends, so we need to define them
-#  include <initguid.h>
-#endif
 #include <audioclient.h>
 #include <functiondiscoverykeys_devpkey.h>
 #include <mmdeviceapi.h>
@@ -57,7 +53,7 @@ public:
         }
 
         return S_OK;
-    };
+    }
 
     HRESULT STDMETHODCALLTYPE OnDeviceRemoved(LPCWSTR deviceID) override
     {
@@ -232,7 +228,7 @@ QList<QAudioDevice> QWindowsAudioDevices::availableDevices(QAudioDevice::Mode mo
         if (!deviceId)
             continue;
 
-        QMaybe<PropertyStoreHelper> props = PropertyStoreHelper::open(device);
+        q23::expected<PropertyStoreHelper, QString> props = PropertyStoreHelper::open(device);
         if (!props) {
             qWarning() << "OpenPropertyStore failed" << props.error();
             continue;
