@@ -89,6 +89,7 @@ private:
     bool processCallback() noexcept QT_MM_NONBLOCKING;
 
     void handleAudioClientError();
+    void joinWorkerThread();
 
     ComPtr<IAudioClient3> m_audioClient;
     ComPtr<IAudioRenderClient> m_renderClient;
@@ -111,8 +112,7 @@ private:
 
     QAudioFormat m_hostFormat;
     std::unique_ptr<char[]> m_preallocatedBuffer;
-    std::unique_ptr<std::pmr::memory_resource> m_pmrBufferResource;
-    std::unique_ptr<std::pmr::memory_resource> m_pmrPoolResource;
+    std::unique_ptr<std::pmr::memory_resource> m_memoryResource;
     std::unique_ptr<QWindowsResampler> m_resampler;
 };
 
