@@ -34,7 +34,7 @@ public:
 
 protected:
     void onActiveChanged(bool active) override;
-    void onCameraDeviceChanged(const QCameraDevice &device) override;
+    void onCameraDeviceChanged(const QCameraDevice &, const QCameraFormat &) override;
     bool tryApplyCameraFormat(const QCameraFormat&) override;
 
 private:
