@@ -10,14 +10,15 @@
 #include <QtCore/qdatetime.h>
 #include <QtCore/qlocale.h>
 #include <QtCore/qurl.h>
+#include <QtCore/private/qcore_mac_p.h>
 #include <QImage>
 #include <QtMultimedia/qvideoframe.h>
 
 #if __has_include(<AppKit/AppKit.h>)
-#include <AppKit/AppKit.h>
+#import <AppKit/AppKit.h>
 #endif
 
-#include <CoreFoundation/CoreFoundation.h>
+#import <CoreFoundation/CoreFoundation.h>
 
 QT_USE_NAMESPACE
 
@@ -110,12 +111,14 @@ const AVMetadataIDs keyToAVMetaDataID[] = {
     // Orientation
     { nil, nil, AVMetadataIdentifierQuickTimeMetadataVideoOrientation, nil, nil, nil },
     // Resolution
+    { nil, nil, nil, nil, nil, nil },
+    // HasHdrContent
     { nil, nil, nil, nil, nil, nil }
 };
 
 static AVMetadataIdentifier toIdentifier(QMediaMetaData::Key key, AVMetadataKeySpace keySpace)
 {
-    static_assert(sizeof(keyToAVMetaDataID)/sizeof(AVMetadataIDs) == QMediaMetaData::Key::Resolution + 1);
+    static_assert(sizeof(keyToAVMetaDataID) / sizeof(AVMetadataIDs) == QMediaMetaData::NumMetaData);
 
     AVMetadataIdentifier identifier = nil;
     if ([keySpace isEqualToString:AVMetadataKeySpaceiTunes]) {
@@ -132,7 +135,7 @@ static AVMetadataIdentifier toIdentifier(QMediaMetaData::Key key, AVMetadataKeyS
 
 static std::optional<QMediaMetaData::Key> toKey(AVMetadataItem *item)
 {
-    static_assert(sizeof(keyToAVMetaDataID)/sizeof(AVMetadataIDs) == QMediaMetaData::Key::Resolution + 1);
+    static_assert(sizeof(keyToAVMetaDataID) / sizeof(AVMetadataIDs) == QMediaMetaData::NumMetaData);
 
     // The item identifier may be different than the ones we support,
     // so check by common key first, as it will get the metadata
@@ -181,7 +184,7 @@ static std::optional<QMediaMetaData::Key> toKey(AVMetadataItem *item)
         itemKeySpace = ID3;
     }
 
-    for (int key = 0; key < QMediaMetaData::Resolution + 1; key++) {
+    for (int key = 0; key < QMediaMetaData::NumMetaData; key++) {
         AVMetadataIdentifier idForKey = nil;
         switch (itemKeySpace) {
         case iTunes:
@@ -250,23 +253,38 @@ QMediaMetaData AVFMetaData::fromAssetTrack(AVAssetTrack *asset)
             if (languageCode) {
                 // languageCode is encoded as ISO 639-2, which QLocale does not handle.
                 // Convert it to 639-1 first.
-                auto id = CFLocaleCreateCanonicalLanguageIdentifierFromString(kCFAllocatorDefault,
-                                                                            (__bridge CFStringRef)languageCode);
-                QString lang = QString::fromCFString(id);
-                CFRelease(id);
-                metadata.insert(QMediaMetaData::Language, QLocale::codeToLanguage(lang));
+                QCFString lang = CFLocaleCreateCanonicalLanguageIdentifierFromString(
+                        kCFAllocatorDefault, (__bridge CFStringRef)languageCode);
+                metadata.insert(QMediaMetaData::Language, QLocale::codeToLanguage(QString{ lang }));
             }
         }
     }
     if ([asset.mediaType isEqualToString:AVMediaTypeVideo]) {
         // add orientation
         if (metadata.value(QMediaMetaData::Orientation).isNull()) {
-            QVideo::RotationAngle angle = QVideo::Rotation0;
+            QtVideo::Rotation angle = QtVideo::Rotation::None;
             bool mirrored;
             AVFMediaPlayer::videoOrientationForAssetTrack(asset, angle, mirrored);
             Q_UNUSED(mirrored);
             metadata.insert(QMediaMetaData::Orientation, int(angle));
         }
+
+        // add HDR content
+        if (metadata.value(QMediaMetaData::HasHdrContent).isNull()) {
+            auto hasHdrContent = false;
+
+            NSArray *formatDescriptions = [asset formatDescriptions];
+            for (id formatDescription in formatDescriptions) {
+                NSDictionary *extensions = (__bridge NSDictionary *)CMFormatDescriptionGetExtensions((CMFormatDescriptionRef)formatDescription);
+                NSString *transferFunction = extensions[(__bridge NSString *)kCMFormatDescriptionExtension_TransferFunction];
+                if ([transferFunction isEqualToString:(__bridge NSString *)kCVImageBufferTransferFunction_SMPTE_ST_2084_PQ]) {
+                    hasHdrContent = true;
+                    break;
+                }
+            }
+
+            metadata.insert(QMediaMetaData::HasHdrContent, hasHdrContent);
+        }
     }
     return metadata;
 }
