@@ -9,11 +9,12 @@
 
 QT_BEGIN_NAMESPACE
 
-QGStreamerCustomAudioDeviceInfo::QGStreamerCustomAudioDeviceInfo(
-        const QByteArray &gstreamerPipeline, QAudioDevice::Mode mode)
+QGStreamerCustomAudioDeviceInfo::QGStreamerCustomAudioDeviceInfo(const QByteArray &gstreamerPipeline,
+                                                                 QAudioDevice::Mode mode)
     : QAudioDevicePrivate{
           gstreamerPipeline,
           mode,
+          QString::fromUtf8(gstreamerPipeline),
       }
 {
 }
@@ -23,7 +24,7 @@ QAudioDevice qMakeCustomGStreamerAudioInput(const QByteArray &gstreamerPipeline)
     auto deviceInfo = std::make_unique<QGStreamerCustomAudioDeviceInfo>(gstreamerPipeline,
                                                                         QAudioDevice::Mode::Input);
 
-    return deviceInfo.release()->create();
+    return QAudioDevicePrivate::createQAudioDevice(std::move(deviceInfo));
 }
 
 QAudioDevice qMakeCustomGStreamerAudioOutput(const QByteArray &gstreamerPipeline)
@@ -31,7 +32,7 @@ QAudioDevice qMakeCustomGStreamerAudioOutput(const QByteArray &gstreamerPipeline
     auto deviceInfo = std::make_unique<QGStreamerCustomAudioDeviceInfo>(gstreamerPipeline,
                                                                         QAudioDevice::Mode::Output);
 
-    return deviceInfo.release()->create();
+    return QAudioDevicePrivate::createQAudioDevice(std::move(deviceInfo));
 }
 
 bool isCustomAudioDevice(const QAudioDevicePrivate *device)
@@ -41,7 +42,7 @@ bool isCustomAudioDevice(const QAudioDevicePrivate *device)
 
 bool isCustomAudioDevice(const QAudioDevice &device)
 {
-    return isCustomAudioDevice(device.handle());
+    return isCustomAudioDevice(QAudioDevicePrivate::handle(device));
 }
 
 QT_END_NAMESPACE
