@@ -1,8 +1,10 @@
 // Copyright (C) 2021 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
-#include <QtTest/QtTest>
+#include <QtTest/qtest.h>
+#include <QtTest/qsignalspy.h>
 #include <QtGui/QImageReader>
+#include <QtCore/qrandom.h>
 #include <QtCore/qurl.h>
 #include <QDebug>
 #include <QVideoSink>
