@@ -1,12 +1,18 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
-#include <QtTest/QtTest>
+#include <QtTest/qsignalspy.h>
+#include <QtTest/qtest.h>
+#include <QtTest/qtesteventloop.h>
 #include <QtMultimedia/qaudio.h>
 #include <QtMultimedia/qaudiodevice.h>
 #include <QtMultimedia/qmediadevices.h>
 #include <QtMultimedia/qsoundeffect.h>
 
+#include <QtMultimedia/private/qsoundeffectsynchronous_p.h>
+#include <QtMultimedia/private/qsoundeffectwithplayer_p.h>
+#include <QtMultimedia/private/qsamplecache_p.h>
+
 using namespace Qt::Literals;
 
 class tst_QSoundEffect : public QObject
@@ -42,11 +48,25 @@ private slots:
     void setAudioDevice_emitsSignalsInExpectedOrder_data();
     void setAudioDevice_emitsSignalsInExpectedOrder();
 
+    void testQSoundEffectVoiceMonoToStereo();
+    void testQSoundEffectVoiceStereoToMono();
+    void testQSoundEffectVoiceSameChannels();
+    void testQSoundEffectVoiceWithVolume();
+    void testQSoundEffectVoiceMuted();
+    void testQSoundEffectVoiceLooping();
+
 private:
     QSoundEffect* sound;
     QUrl url; // test.wav: pcm_s16le, 48000 Hz, stereo, s16
     QUrl url2; // test_tone.wav: pcm_s16le, 44100 Hz, mono
     QUrl urlCorrupted; // test_corrupted.wav: corrupted
+
+    SharedSamplePtr createTestSample(QSpan<const float> floats, const QAudioFormat &format)
+    {
+        QByteArray data(reinterpret_cast<const char *>(floats.data()),
+                        floats.size() * sizeof(float));
+        return std::make_shared<QSample>(data, format);
+    }
 };
 
 void tst_QSoundEffect::init()
@@ -128,7 +148,7 @@ void tst_QSoundEffect::testLooping()
     QTestEventLoop::instance().enterLoop(3);
 
     QTRY_COMPARE(sound->loopsRemaining(), 0);
-    QVERIFY(readSignal_Remaining.size() == 4);
+    QCOMPARE(readSignal_Remaining.size(), 4);
     QTRY_VERIFY(!sound->isPlaying());
 
     // QTBUG-36643 (setting the loop count while playing should work)
@@ -218,6 +238,8 @@ void tst_QSoundEffect::testMuting()
 
 void tst_QSoundEffect::testPlaying()
 {
+    QSignalSpy playingChangedSignal(sound, &QSoundEffect::playingChanged);
+
     sound->setLoopCount(QSoundEffect::Infinite);
     sound->setVolume(0.1f);
     //valid source
@@ -226,14 +248,18 @@ void tst_QSoundEffect::testPlaying()
     sound->play();
     QTestEventLoop::instance().enterLoop(1);
     QTRY_COMPARE(sound->isPlaying(), true);
+    QCOMPARE(playingChangedSignal.size(), 1);
     sound->stop();
+    QCOMPARE(playingChangedSignal.size(), 2);
 
     //empty source
+    playingChangedSignal.clear();
     sound->setSource(QUrl());
     QTestEventLoop::instance().enterLoop(1);
     sound->play();
     QTestEventLoop::instance().enterLoop(1);
     QTRY_COMPARE(sound->isPlaying(), false);
+    QCOMPARE(playingChangedSignal.size(), 0);
 
     //invalid source
     sound->setSource(QUrl((QLatin1String("invalid source"))));
@@ -395,7 +421,7 @@ void tst_QSoundEffect::testCorruptFile()
 {
     using namespace Qt::Literals;
     auto expectedMessagePattern =
-            QRegularExpression(uR"(^QSoundEffect\(qaudio\): Error decoding source .*$)"_s);
+            QRegularExpression(uR"(^QSoundEffect.*: Error decoding source .*$)"_s);
 
     for (int i = 0; i < 10; i++) {
         QSignalSpy statusSpy(sound, &QSoundEffect::statusChanged);
@@ -421,9 +447,16 @@ void tst_QSoundEffect::setAudioDevice_emitsSignalsInExpectedOrder_data()
     QTest::addColumn<bool>("while_playing");
     QTest::addColumn<bool>("with_source");
     QTest::addColumn<QStringList>("expectedSignals");
-    QTest::addRow("while_playing")
-            << true << true
-            << QStringList{ u"playingChanged"_s, u"playingChanged"_s, u"audioDeviceChanged"_s };
+    QSoundEffect sfx(this);
+
+    if (dynamic_cast<QSoundEffectPrivateSynchronous *>(QSoundEffectPrivate::get(&sfx))) {
+        QTest::addRow("while_playing")
+                << true << true
+                << QStringList{ u"playingChanged"_s, u"playingChanged"_s, u"audioDeviceChanged"_s };
+    } else {
+        QTest::addRow("while_playing")
+                << true << true << QStringList{ u"playingChanged"_s, u"audioDeviceChanged"_s };
+    }
     QTest::addRow("while_stopped, with source")
             << false << true << QStringList{ u"audioDeviceChanged"_s };
     QTest::addRow("while_stopped, without source")
@@ -470,7 +503,6 @@ void tst_QSoundEffect::setAudioDevice_emitsSignalsInExpectedOrder()
         return outputs[1];
     }();
 
-    QVERIFY(nonDefaultDevice != outputs.at(1));
     emittedSignals.clear();
 
     // Act
@@ -481,6 +513,161 @@ void tst_QSoundEffect::setAudioDevice_emitsSignalsInExpectedOrder()
     QCOMPARE(emittedSignals, expectedSignals);
 }
 
+using QtMultimediaPrivate::QSoundEffectVoice;
+using QtMultimediaPrivate::VoiceId;
+
+void tst_QSoundEffect::testQSoundEffectVoiceMonoToStereo()
+{
+    std::array<float, 2> sampleData = { 1.0f, 0.5f };
+    QAudioFormat sampleFormat;
+    sampleFormat.setSampleFormat(QAudioFormat::Float);
+    sampleFormat.setChannelCount(1);
+    sampleFormat.setSampleRate(44100);
+    auto sample = createTestSample(sampleData, sampleFormat);
+
+    QAudioFormat engineFormat = sampleFormat;
+    engineFormat.setChannelCount(2);
+
+    QSoundEffectVoice voice(VoiceId{ 0 }, sample, 1.0f, false, 1, engineFormat);
+
+    std::array<float, 4> buffer = { 0.0f, 0.0f, 0.0f, 0.0f };
+    qsizetype played = voice.playVoice(buffer);
+
+    QCOMPARE(played, 2);
+    QCOMPARE(buffer[0], 1.0f);
+    QCOMPARE(buffer[1], 1.0f);
+    QCOMPARE(buffer[2], 0.5f);
+    QCOMPARE(buffer[3], 0.5f);
+}
+
+void tst_QSoundEffect::testQSoundEffectVoiceStereoToMono()
+{
+    std::array<float, 4> sampleData = { 1.0f, 0.2f, 0.5f, 0.3f };
+    QAudioFormat sampleFormat;
+    sampleFormat.setSampleFormat(QAudioFormat::Float);
+    sampleFormat.setChannelCount(2);
+    sampleFormat.setSampleRate(44100);
+    auto sample = createTestSample(sampleData, sampleFormat);
+
+    QAudioFormat engineFormat = sampleFormat;
+    engineFormat.setChannelCount(1);
+
+    QSoundEffectVoice voice(VoiceId{ 0 }, sample, 1.0f, false, 1, engineFormat);
+
+    std::array<float, 2> buffer = { 0.0f, 0.0f };
+    qsizetype played = voice.playVoice(buffer);
+
+    QCOMPARE(played, 2);
+    QCOMPARE(buffer[0], 0.6f);
+    QCOMPARE(buffer[1], 0.4f);
+}
+
+void tst_QSoundEffect::testQSoundEffectVoiceSameChannels()
+{
+    std::array<float, 2> sampleData = { 1.0f, 0.5f };
+    QAudioFormat sampleFormat;
+    sampleFormat.setSampleFormat(QAudioFormat::Float);
+    sampleFormat.setChannelCount(1);
+    sampleFormat.setSampleRate(44100);
+    auto sample = createTestSample(sampleData, sampleFormat);
+
+    QAudioFormat engineFormat = sampleFormat; // Same channels
+
+    QSoundEffectVoice voice(VoiceId{ 0 }, sample, 1.0f, false, 1, engineFormat);
+
+    std::array<float, 2> buffer = { 0.0f, 0.0f };
+    qsizetype played = voice.playVoice(buffer);
+
+    QCOMPARE(played, 2);
+    QCOMPARE(buffer[0], 1.0f);
+    QCOMPARE(buffer[1], 0.5f);
+}
+
+void tst_QSoundEffect::testQSoundEffectVoiceWithVolume()
+{
+    std::array<float, 2> sampleData = { 1.0f, 0.5f };
+    QAudioFormat sampleFormat;
+    sampleFormat.setSampleFormat(QAudioFormat::Float);
+    sampleFormat.setChannelCount(1);
+    sampleFormat.setSampleRate(44100);
+    auto sample = createTestSample(sampleData, sampleFormat);
+
+    QAudioFormat engineFormat = sampleFormat;
+    engineFormat.setChannelCount(2);
+
+    QSoundEffectVoice voice(VoiceId{ 0 }, sample, 0.5f, false, 1, engineFormat);
+
+    std::array<float, 4> buffer = { 0.0f, 0.0f, 0.0f, 0.0f };
+    qsizetype played = voice.playVoice(buffer);
+
+    QCOMPARE(played, 2);
+    QCOMPARE(buffer[0], 0.5f);
+    QCOMPARE(buffer[1], 0.5f);
+    QCOMPARE(buffer[2], 0.25f);
+    QCOMPARE(buffer[3], 0.25f);
+}
+
+void tst_QSoundEffect::testQSoundEffectVoiceMuted()
+{
+    std::array<float, 2> sampleData = { 1.0f, 0.5f };
+    QAudioFormat sampleFormat;
+    sampleFormat.setSampleFormat(QAudioFormat::Float);
+    sampleFormat.setChannelCount(1);
+    sampleFormat.setSampleRate(44100);
+    auto sample = createTestSample(sampleData, sampleFormat);
+
+    QAudioFormat engineFormat = sampleFormat;
+    engineFormat.setChannelCount(2);
+
+    QSoundEffectVoice voice(VoiceId{ 0 }, sample, 1.0f, true, 1, engineFormat);
+
+    std::array<float, 4> buffer = { 1.0f, 1.0f, 1.0f, 1.0f }; // Non-zero to check muting
+    qsizetype played = voice.playVoice(buffer);
+
+    QCOMPARE(played, 2);
+    QCOMPARE(buffer[0], 0.0f);
+    QCOMPARE(buffer[1], 0.0f);
+    QCOMPARE(buffer[2], 0.0f);
+    QCOMPARE(buffer[3], 0.0f);
+}
+
+void tst_QSoundEffect::testQSoundEffectVoiceLooping()
+{
+    std::array<float, 2> sampleData = { 1.0f, 0.5f };
+    QAudioFormat sampleFormat;
+    sampleFormat.setSampleFormat(QAudioFormat::Float);
+    sampleFormat.setChannelCount(1);
+    sampleFormat.setSampleRate(44100);
+    auto sample = createTestSample(sampleData, sampleFormat);
+
+    QAudioFormat engineFormat = sampleFormat;
+
+    QSoundEffectVoice voice(VoiceId{ 0 }, sample, 1.0f, false, 2, engineFormat);
+
+    // Buffer large enough for 2 loops: 2 frames * 1 channel * 2 loops = 4 samples
+    std::array<float, 4> buffer = { 0.0f, 0.0f, 0.0f, 0.0f };
+
+    // Check initial
+    QCOMPARE(voice.loopsRemaining(), 2);
+
+    // Spy on signal
+    using namespace QtMultimediaPrivate;
+    QSignalSpy spy(&voice.m_currentLoopChanged, &QAutoResetEvent::activated);
+
+    // Play
+    VoicePlayResult result = voice.play(buffer);
+
+    QCOMPARE(result, VoicePlayResult::Finished);
+    QCOMPARE(voice.loopsRemaining(), 0);
+    QTRY_VERIFY(spy.size() == 1); // Wait for signal
+
+    // Check buffer: should have the sample repeated
+    QCOMPARE(buffer[0], 1.0f);
+    QCOMPARE(buffer[1], 0.5f);
+    QCOMPARE(buffer[2], 1.0f);
+    QCOMPARE(buffer[3], 0.5f);
+}
+
 QTEST_MAIN(tst_QSoundEffect)
 
 #include "tst_qsoundeffect.moc"
